package com.hkrt.hkpossdktest;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;
import pub.devrel.easypermissions.EasyPermissions;

import android.Manifest;
import android.content.Intent;
import android.os.Bundle;
import android.text.TextUtils;
import android.widget.Button;
import android.widget.TextView;
import android.widget.Toast;

import com.google.gson.Gson;
import com.hkrt.hk_saas_pos_standard_sdk.TestActivity;
import com.hkrt.hk_saas_pos_standard_sdk.base.BaseController;
import com.hkrt.hk_saas_pos_standard_sdk.base.HkSdkCallback;
import com.hkrt.hk_saas_pos_standard_sdk.factory.BaseSdkFactory;
import com.hkrt.hk_saas_pos_standard_sdk.factory.LandiSdkFactory;
import com.hkrt.hk_saas_pos_standard_sdk.factory.NewlandSdkFactory;
import com.hkrt.hk_saas_pos_standard_sdk.factory.SunmiSdkFactory;
import com.hkrt.hk_saas_pos_standard_sdk.utils.TranDetail;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class MainActivity extends AppCompatActivity implements HkSdkCallback, EasyPermissions.PermissionCallbacks{

    TextView tvHintText;
    Button btnConsume, btnRevoke, btnRefund, btnSign,btnMerchant, btnQuery;
    TranDetail tranDetail;
    private static final int PERMISSION_REQUESTCODE_CONSUME = 101;
    private static final int PERMISSION_REQUESTCODE_REVOKE = 102;
    private static final int PERMISSION_REQUESTCODE_REFUND = 103;
    private boolean paramIsDownloadSuccess = false;
    BaseController sdkController;
    String outTradeNo;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        tvHintText = findViewById(R.id.tvHint);
        btnConsume = findViewById(R.id.btnConsume);
        btnRevoke = findViewById(R.id.btnRevoke);
        btnRefund = findViewById(R.id.btnRefund);
        btnSign = findViewById(R.id.btnSign);
        btnMerchant = findViewById(R.id.btnMerchant);
        btnQuery = findViewById(R.id.btnQuery);


        // 联迪A8
        BaseSdkFactory factory = new LandiSdkFactory();
        // 新大陆N910
//        BaseSdkFactory factory = new NewlandSdkFactory();
        // 商米P2
//        BaseSdkFactory factory = new SunmiSdkFactory();

        sdkController = factory.getSdkController(this,this);
        sdkController.init(this);

        btnConsume.setOnClickListener(v -> {
            if (paramIsDownloadSuccess) {
                requestTradeNeedPermission(PERMISSION_REQUESTCODE_CONSUME);
            } else {
                Toast.makeText(this, "请先等待设备参数下载完成。", Toast.LENGTH_SHORT).show();
            }

        });

        btnRevoke.setOnClickListener(v -> {
            if (paramIsDownloadSuccess) {
                requestTradeNeedPermission(PERMISSION_REQUESTCODE_REVOKE);
            }else {
                Toast.makeText(this, "请先等待设备参数下载完成。", Toast.LENGTH_SHORT).show();
            }

        });
        btnRefund.setOnClickListener(v -> {
            if (paramIsDownloadSuccess) {
                requestTradeNeedPermission(PERMISSION_REQUESTCODE_REFUND);

            }else {
                Toast.makeText(this, "请先等待设备参数下载完成。", Toast.LENGTH_SHORT).show();
            }

        });

        btnMerchant.setOnClickListener(v -> {
            sdkController.getMerchantInfo();
        });

        btnQuery.setOnClickListener(v -> {
            if (TextUtils.isEmpty(outTradeNo)) {
                Toast.makeText(this, "请先消费交易。", Toast.LENGTH_SHORT).show();
                return;
            }
            sdkController.queryCardPayDetail(outTradeNo);
        });
        tvHintText.setText("SDK初始化中。。。");

    }

    @Override
    public void onPermissionsGranted(int requestCode, @NonNull List<String> perms) {

        toTrade(requestCode);
    }

    @Override
    public void onPermissionsDenied(int requestCode, @NonNull List<String> perms) {

        Toast.makeText(this, "请为sdk开启读写、定位权限以保证各项功能可正常使用", Toast.LENGTH_SHORT).show();

    }

    private void toTrade(int requestCode) {

        if (requestCode == PERMISSION_REQUESTCODE_CONSUME) {

            // 银行卡消费
            Map<String,Object> consumeParam = new HashMap<>();


            outTradeNo = "OUT_TEST" + System.currentTimeMillis() + "";
            consumeParam.put("outTradeNo",outTradeNo);
            consumeParam.put("totalAmount","0.01");
            consumeParam.put("remark","sdk交易测试");
            consumeParam.put("notifyUrl","交易结果异步通知地址");
            consumeParam.put("ledger_flag","");
            consumeParam.put("ledger_relation",new ArrayList<Map>());
            consumeParam.put("printFlag","1");

            sdkController.toConsume(this,consumeParam);

        }else  if (requestCode == PERMISSION_REQUESTCODE_REVOKE) {
            // 银行卡消费撤销
            if (tranDetail == null || TextUtils.isEmpty(tranDetail.getTradeId())) {
                Toast.makeText(this, "请先进行消费", Toast.LENGTH_SHORT).show();
                return;
            }

            Map<String, Object> revokeParam = new HashMap<>();
            revokeParam.put("outTradeNo", "OUT_TESTRevoke" + System.currentTimeMillis() + "");
            revokeParam.put("tradeId", tranDetail.getTradeId());
            revokeParam.put("notifyUrl", "交易结果异步通知地址");
            revokeParam.put("printFlag", "2");

            sdkController.toRevoke(this, revokeParam);

        }else  if (requestCode == PERMISSION_REQUESTCODE_REFUND) {
            // 银行卡消费退货
            if (tranDetail == null || TextUtils.isEmpty(tranDetail.getTradeId())) {
                Toast.makeText(this, "请先进行消费", Toast.LENGTH_SHORT).show();
                return;
            }
            Map<String,Object> refundParam = new HashMap<>();
            refundParam.put("outTradeNo","OUT_TESTRefund" + System.currentTimeMillis() + "");
            refundParam.put("refundAmount","0.01");
            refundParam.put("tradeId",tranDetail.getTradeId());
            refundParam.put("remark","sdk交易测试");
            refundParam.put("notifyUrl","交易结果异步通知地址");
            refundParam.put("printFlag","2");

            sdkController.toRefund(this,refundParam);
        }

    }

    private void requestTradeNeedPermission(int requestCode) {

        String[] perms = {Manifest.permission.CAMERA,Manifest.permission.READ_EXTERNAL_STORAGE, Manifest.permission.WRITE_EXTERNAL_STORAGE,Manifest.permission.ACCESS_FINE_LOCATION};
        if (EasyPermissions.hasPermissions(this,perms)) {
            // 已经申请过权限，做想做的事
            toTrade(requestCode);
        } else {
            // 没有申请过权限，现在去申请
            EasyPermissions.requestPermissions(this,"请为sdk开启读写、定位权限以保证各项功能可正常使用",requestCode,perms);
        }
    }


    @Override
    public void onError(String errorCode, String errorMsg) {
        runOnUiThread(() -> {
            tvHintText.setText(errorCode + ":" + errorMsg);
        });

    }

    @Override
    public void onInitSuccess() {
        runOnUiThread(() -> {
            tvHintText.setText("SDK初始化完成，开始设备下载参数。。。");
            Toast.makeText(this, "设备sdk初始化成功", Toast.LENGTH_SHORT).show();

            sdkController.toDeviceSignIn(this);
        });


    }

    @Override
    public void onSignInSuccess() {
        runOnUiThread(() -> {
            tvHintText.setText("设备下载参数完成。。。");
            paramIsDownloadSuccess = true;
        });


    }

    @Override
    public void onGetMerchantInfo(Map<String, Object> map) {
        runOnUiThread(() -> {
            tvHintText.setText("获取商户信息成功。。。返回数据：" + map.toString());
        });

    }

    @Override
    public void onConsumeSuccess(String result) {

        runOnUiThread(() -> {
            tvHintText.setText("消费成功。。。返回数据：" + result);

            tranDetail = new Gson().fromJson(result, TranDetail.class);
        });

    }

    @Override
    public void onRevokeSuccess(String result) {

        runOnUiThread(() -> {
            tvHintText.setText("撤销成功。。。返回数据：" + result);
        });

    }

    @Override
    public void onRefundSuccess(String result) {
        runOnUiThread(() -> {
            tvHintText.setText("退货成功。。。返回数据：" + result);
        });


    }

    @Override
    public void onGetCardPayDetail(String result) {
        runOnUiThread(() -> {
            tvHintText.setText("交易查询。。。返回数据：" + result);
        });
    }

    @Override
    public void onTicketUploadSuccess() {
//        tvHint.setText("上传小票成功。。。");

    }

    @Override
    public void onTicketPrintSuccess() {
        runOnUiThread(() -> {
            tvHintText.setText("打印签购单成功。。。");
        });


    }

}